/* 
   StarDate - calculate the stardate
   $Id: StarDate.java,v 1.1 2003/04/17 17:31:39 ranga Exp $ 
*/

import java.text.*;
import java.util.Date;

public class StarDate {

  private Date sdate;
  private DecimalFormat df;

  public StarDate() { 
    sdate = new Date() ; 
    df = new DecimalFormat("0.00");
  }

  public String toString() {
    sdate.setTime(System.currentTimeMillis());
    return df.format(calcStarDate(sdate));
  }
	
  /* Returns the stardate for the given date */

  private double calcStarDate(Date d) {
    double jd, gmt, gmst;
    
    jd = julian(d.getMonth() + 1,
		d.getDate(),
		d.getYear() + 1900);

    gmt = (double)(d.getHours() + 
		   d.getMinutes()/60.0 + 
		   d.getSeconds()/3600.0);

    gmst = sidereal(gmt,jd,d.getYear());

    return (double)(jd + gmst/24.0);		
  }

  /* Returns the New Epoch Julian day for a given Gregorian day 
   * Based on subroutine julian_day from stardate.c by Jay Windley */

  private double julian (int month, int day, int year) {
    int loc_month,loc_year,a,b,c,d;

    loc_month = month;
    loc_year = year;

    /* Correct for March based B.C. year */

    if (year < 0) loc_year++;

    if (month < 3) {
      loc_month = month + 12;
      loc_year--;
    }

    /* Set up a & b for Julian to Gregorian calendar change */

    if (year < 1582) {
      b = 0;
    } else if (year == 1582) {
      if (month < 10) {
	b = 0;
      } else if (month == 10) {
	if (day < 15) {
	  b = 0;
	} else {
	  a = loc_year / 100;
	  b = 2 - a + (a/4);
	}
      } else {
	a = loc_year / 100;
	b = 2 - a + (a/4);
      }
    } else {
      a = loc_year / 100;
      b = 2 - a + (a/4);
    }

    /* Fudge factors for year and month */

    c = (int)(365.25 * loc_year) - 694025;
    d = (int)(30.6001 * (loc_month + 1));
		
    /* return the julian day */

    return (double)((b + c + d + day) - 0.5);	

  }

  /* Returns the Greenwich Mean Sidereal Time for the given Greenwich 
   * Mean Solar Time on the given New Epoch Julian Day 
   * Based on subroutine julian_day from stardate.c by Jay Windley */ 

  private double sidereal (double gmt, double jd, int gyear) {

    double dayfudge, yrfudge, hrfudge;
    double initjd, jhund, time0, lst;

    /* Find Julian century at start of year */

    initjd = julian(1,0,gyear);
    jhund = initjd / 36525.0;

    /* Apply polynomial correction change */
	
    dayfudge = 6.6460656 + (0.051262 + (jhund * 0.00002581)) * jhund;
    yrfudge = 2400.0 * (jhund - ((double)(gyear - 1900.0) / 100.0));
    hrfudge = 24.0 - dayfudge - yrfudge;
    time0 = (jd - initjd) * 0.0657098 - hrfudge;

    /* Adjust to modulo 24 hrs */
    lst = (gmt * 1.002737908) + time0;
		
    while (lst < 0) lst += 24.0;
    while (lst > 24.0) lst -= 24.0;

    return lst;
  }
}

